/*************************************************************************
Author:   	$Author: dennis $
File:     	$HeadURL: file:///home/dennis/svn-store/avr-source/mptry-fat32/main.c $
Date:  		$Date: 2008-10-18 12:20:15 +0200 (Sa, 18. Okt 2008) $
Revision: 	$Revision: 172 $
Id: 		$Id: main.c 172 2008-10-18 10:20:15Z dennis $
Licence:	GNU General Public License
 *************************************************************************/
/**
 *  @defgroup mptry MP3 Player
 *  @code #include <blabla.h> @endcode
 *
 *  @brief MP3 Player application
 *
 *  This is a mp3 player.
 *
 *  @author Dennis Dingeldein  http://www.dingeldein-online.de
 */

/*! \mainpage MP3 Player application with AVR
 *
 * \section intro_sec Introduction
 * These files implement a ....
 *
 * \section install_sec Hardware
 * To run the software, it must be compiled according to your hardware.
 * See ...
 *
 * \section setup
 * ...
 *
 *
 * \section Software
 * This code implements a kind of a
 *
 * \section Files
 * ...
 *
 *
 * \section Licence
 * This code can be used according to GNU General Public License.
 *
 * \author
 * Dennis Dingeldein
 */

//#include "fat16.h"
//#include "partition.h"

#include "config.h"

#include "sd_mmc.h" // for the sd_init() function, nothing else

#include "vs1011.h"
#include "uart.h"

#include <string.h>
#include <stdio.h>
#include <avr/interrupt.h>

#include "delay.h"
#include <util/delay.h>

#include "spi.h"
#include "vs1011.h"

#include "timer16.h"

#include "lcd_cx65.h"

#include "rc5.h"

#include "ff.h"

#include "player.h"

#define DEBUG 1
#define UART_BAUDRATE 19200

#define LINE_GREETING 	1
#define LINE_ARTIST 	3
#define LINE_TITLE 	4
#define LINE_STATE		5
#define LINE_BROWSE	6

uint8_t print_disk_info( void );
void input_process( void );

// global player var
volatile Player_t player;

// buffer size for mp3 data read from sd/mmc.
// if you define BUFSIZE too large, the remaining memory will not be enough for executing the player code.
// for a mega32, even 256 bytes are too large.
#define BUFSIZE 128

char sbuf[32]; // utility buffer for printing strings
uint8_t buffer[BUFSIZE]; // buffer fpr mp3 data read from SD/MMC
uint16_t buffer_ptr;


uint8_t print_id3_tags( FIL *file ) {
	int32_t offset = -128L + file->fsize;
	uint16_t nbytes;

	// ID3V1 tags are at the last 128 bytes of a MP3 file - seek to that position
	if (f_lseek( file, offset ) != FR_OK) {
		return 0;
	}

	// read magic value of length 3 bytes, it should contain value "TAG"
	if ((f_read( file, buffer, 3, &nbytes) != FR_OK) || (nbytes!=3)) {
		// TODO handle read failure
	}

	// check for magic bytes
	if (strncmp( (char*)buffer, "TAG", 3) != 0)
		return 1; // not an error, but no tags are there

	// if we arrive here, ID3V1 tags may exist

	// title
	if ((f_read( file, buffer, 30, &nbytes) != FR_OK) || (nbytes!=30)) {
		// TODO handle read failure
		return 1;
	}
	buffer[22]='\000';
	print_str(0,(LINE_TITLE*CHAR_H) % (12*14), "                      ");
	print_str(0,(LINE_TITLE*CHAR_H) % (12*14), (char*)buffer);
	// artist
	if ((f_read( file, buffer, 30, &nbytes) != FR_OK) || (nbytes!=30)) {
		// TODO handle read failure
		return 1;
	}
	print_str(0,(LINE_ARTIST*CHAR_H) % (12*14), "                      ");
	buffer[22]='\000';
	print_str(0,(LINE_ARTIST*CHAR_H) % (12*14), (char*)buffer);

#ifdef COMPLETE_TAGS
	// album
	if ((f_read( file, buffer, 30, &nbytes) != FR_OK) || (nbytes!=30)) {
		// TODO handle read failure
		return 1;
	}

	// year
	if (f_read(file, buffer, 4, &nbytes) != FR_OK || (nbytes!=4)) {
		// TODO handle read failure
		return 1;
	}

	// comment
	if (f_read(file, buffer, 30, &nbytes) != FR_OK || (nbytes!=30)) {
		// TODO handle read failure
		return 1;
	}

	// genre
	if (f_read(file, buffer, 1, &nbytes) != FR_OK || (nbytes!=1)) {
		// TODO handle read failure
		return 1;
	}
#endif
	// set back fp pointers to oiginal value
	offset = 0L;
	if (f_lseek( file, offset ) != FR_OK) {
		return 0;
	}

	return 1;
}

int main() {
	char greeting[]="mptry";
	uint8_t s_old=0; // contains last (time second) value
	uint8_t m=0; // minute value
	uint8_t state_old=0; // contains last state value

	uint8_t ir_code; // IR remote control code
	uint8_t ir_addr; // IR remote control address

	FATFS fs; // FAT filesystem var
	uint8_t ret; // general purpose return value
	DIR dir; // directory var
	FIL fil; // file var
	FILINFO finfo; // file info var
	UINT nbytes;

	// --------------
	// init UART
	// --------------
	uart_init(UART_BAUD_SELECT(UART_BAUDRATE,F_CPU));
	sei();
	uart_puts("\n\rmptry $Revision: 172 $\n\r");

	// debug pin to zero
	//DDRA |= (1 << PA5);
	//PORTA &= ~_BV(PA5);

	// unselect SD card
	unselect_card();

	// --------------
	// init spi
	// --------------
	spi_init();

	// --------------
	// VS1011 init stuff
	// --------------
	// assign pins
	vs1011_port_init();
	vs1011_xcs_deselect();

	uart_puts("starting init vs\n\r");
	// before init of vs1011, dreq should be 0
	loop_until_bit_is_clear(PINA,VS1011_DREQ);
	uart_puts("before hw init vs\n\r");
	// select chip
	vs1011_xcs_select();
	// hw init chip
	vs1011_init();
	uart_puts("after hw init vs\n\r");

	vs1011_xcs_deselect();

	uart_puts("before sw init vs\n\r");
	vs1011_sw_reset();
	uart_puts("after sw init vs\n\r");

	//vs1011_dim_volume(0x00);
	//vs1011_test_sine_on_off();

	// --------------
	// SD/MMC card init stuff
	// --------------
	uart_puts("starting card init\n\r");

	// f_mount tries to initialize the SD/MMC card
	uart_puts("f_mount try\n\r");
	ret = f_mount( 0, &fs );
	sprintf( sbuf, "f_mount ret=%d\n\r", ret);
	uart_puts(sbuf);
	if (ret!=FR_OK) {
		uart_puts("f_mount failed\n\r");
	} else
		uart_puts("f_mount ok\n\r");

#ifdef JUST_TESTING_FOPEN
	uart_puts("f_open try\n\r");
	ret = f_open( &fil, "hddata.txt", FA_READ);
	sprintf( sbuf, "ret=%d\n\r", ret);
	uart_puts(sbuf);
	if (ret!=FR_OK) {
		uart_puts("f_open failed\n\r");
	} else
		uart_puts("f_open ok\n\r");
#endif

#ifdef JUST_TESTING_FREAD
	ret = f_read( &fil, buf, 6, &nbytes );
	uart_putc_hex(nbytes);
	if (ret!=FR_OK) {
		uart_puts("f_read failed\n\r");
	} else
		uart_puts("f_read ok\n\r");
	for (int i=0; i<nbytes; i++ ) {
		uart_putc( buf[i]);
	}
	f_close(&fil);
#endif

#ifdef TEST_READ_SEC_0
	/* test : read sector 0 */
	if (f_read( 0, buffer, 512 ) != FR_OK) {
		uart_puts("MMC/SD read failed\n\r");
		//continue;
	}
	display_buffer( buffer, 512 );
#endif

	// --------------
	// player init stuff
	// --------------
	// if we make it without errors up to here, the "player" itself can be initialized
	// because all player components seems to exist and work

	// init gobal player object
	player.state = S_IDLE;
	player.last_event = EV_NONE;

	// --------------
	// LCD init stuff
	// --------------
	port_init();
	lcd_init_c();
	backcolor=0xFFFF;
	textcolor=0x0000;
	//fill_screen(0x0780);
	fill_screen(backcolor);
	//print_str(0,(LINE_GREETING*CHAR_H) % (12*14),greeting);
	uint16_t c_yellow = color( 0xff, 0xff, 0x00 );
	uint16_t c_red = color( 0xff, 0x00, 0x00 );
	draw_textbox( 50, 0, greeting, c_yellow, c_red );

	//---------------
	// subdir handling
	// --------------

	DIR browse_dir;
	FIL browse_fil;
	FILINFO browse_info;
	char g_path_file[(8+3)*5]; 	// path to currently playing/playable file
	char g_path_dir[(8+3)*5];		// path to currently usable dir (without file name part)
	char g_browse_file[(8+3)*5];	// path to currently browsed file
	char g_browse_dir[(8+3)*5];	// path to currently browsed dir (without file name part)

	// init path vars
	strcpy( g_path_dir, "/" );
	// init subdir path
	strcpy( g_browse_dir, g_path_dir );

	//---------------
	// FAT filesystem init
	// --------------

	ret = f_opendir( &dir, g_path_dir );
	if (ret!=FR_OK) {
		uart_puts("f_opendir failed\n\r");
	} else {
		uart_puts("f_opendir ok\n\r");
	}

	/* print some card information */
	//print_disk_info(player.fs);

	// empty buffer so far
	buffer_ptr=0;

#ifdef JUST_PRINTING_ROOT_DIR
	ret = f_readdir(&dir, &finfo);
	while((ret==FR_OK) && finfo.fname[0]) {
		//uint8_t spaces = sizeof(dir_entry.long_name) - strlen(dir_entry.long_name) + 4;

		uart_puts(finfo.fname);
		uart_putc(finfo.fattrib & AM_DIR ? '/' : ' ');
		//while(spaces--)
		//	uart_putc(' ');
		uart_putdw_dec(finfo.fsize);
		uart_puts("\n\r");
		ret = f_readdir(&dir, &finfo);
	}
	// open again
	ret = f_opendir( &dir, "/" );
	if (ret!=FR_OK) {
		uart_puts("f_opendir failed\n\r");
	} else {
		uart_puts("f_opendir ok\n\r");
	}
#endif

	//------------
	// timer IR init
	//------------
	timer16_init();

	// --------------
	// IR remote control init stuff
	// --------------
	rc5_init(RC5_ALL);

	// draw funny logo
	draw();


	// --------------
	// endless command execution loop
	// --------------
	uint8_t i;
	char *p;

	uart_puts("Entering endless loop...\n\r");
	for (;;) {

		//-------------
		// handle IDLE state
		//-------------
		if (player.state&S_IDLE) {
			// S_IDLE means idle state. For now, we automatically start playing next file from dir
			// search for first/next non-directory (i.e. for first file) in dir
			// and assume that this is a mp3.
			// we will use this as the next file to play
			ret = f_readdir(&dir, &finfo);
			if(ret != FR_OK) {
				uart_puts("error reading dir 1 ");
			}
			while (finfo.fattrib & AM_DIR) {
				ret = f_readdir(&dir, &finfo);
				if(ret != FR_OK) {
					uart_puts("error reading dir 2");
				}
			}

			if (!(finfo.fattrib  & AM_DIR) && (strstr(finfo.fname, ".MP3") != NULL) ) {
				uart_puts("g_path_dir=\"");
				uart_puts(g_path_dir);
				uart_puts("\"\n\r");
				strcpy( g_path_file, g_path_dir);
				// add delimiter except for base path "/"
				if (strlen(g_path_file)>1)
					strcat( g_path_file, "/" );
				strcat( g_path_file, finfo.fname );
				uart_puts("playing: \"");
				uart_puts(g_path_file);
				uart_puts("\"\n\r");
				print_str(0,(LINE_ARTIST*CHAR_H) % (12*14),finfo.fname);

				ret = f_open( &fil, g_path_file, FA_READ );
				if(ret != FR_OK) {
					uart_puts("error opening ");
					uart_puts("file\n\r");
				}
				print_id3_tags( &fil );
			}
			// play song
			uart_puts("try to play mp3\n\r");
			player.state |= S_PLAY;
			player.state&=(~S_IDLE);

			// reset time
			s=0; m=0;
		}

		//----------------
		// playing
		// ---------------
		// if player is playing and VS1011 can take new data
		if ((player.state&S_PLAY) && bit_is_set(PINA,VS1011_DREQ)) {

			// playing state and vs1011 can receive at least 32 bytes

			// read new data from sd/mmc if required
			if (buffer_ptr==0) {
				// buffer is empty, read in a new chunk to buffer
				ret = f_read( &fil, buffer, sizeof(buffer), &nbytes);
				if ((ret != FR_OK) || (nbytes!=sizeof(buffer))) {
					// end of file reached. Add Event EV_EOF.
					player.last_event=EV_EOF;
				}
			}

			// if there is MP3 data from file left put a 32 byte chunk to VS1011
			// activate xDCS
			vs1011_xdcs_select();
			// for each of 32 bytes in buffer ...
			for(uint16_t i = 0; i < 32; ++i) {
				// write single data byte from buffer
				spi_write_byte(buffer[i+buffer_ptr]);
			}
			// deactivate xDCS
			vs1011_xdcs_deselect();

			// adjust buffer_ptr value
			buffer_ptr += 32;
			// reset buffer_ptr if end of buffer was/is reached
			if (buffer_ptr>=sizeof(buffer))
				buffer_ptr=0;
		}

		//----------------
		// print some information to LCD
		//----------------

		// time
		if (s!=s_old) {
			if (s==60) {
				s=0;
				m++;
			}
			// time value
			sprintf( sbuf, "%02d:%02d", m, s);
			print_str(0,(LINE_STATE*CHAR_H) % (12*14),sbuf);
			//uart_puts(sbuf);
			s_old=s;
		}

		// player state
		if (player.state != state_old) {
			state_old = player.state;
			print_str(6*CHAR_W,(LINE_STATE*CHAR_H) % (12*14),"       ");
			if (player.state&S_PLAY)
				print_str(6*CHAR_W,(LINE_STATE*CHAR_H) % (12*14),"playing");
			if (player.state&S_PAUSE)
				print_str(6*CHAR_W,(LINE_STATE*CHAR_H) % (12*14),"pause");
			if (player.state&S_IDLE)
				print_str(6*CHAR_W,(LINE_STATE*CHAR_H) % (12*14),"");
			if (player.state&S_BROWSE)
				print_str((6+7+1)*CHAR_W,(LINE_STATE*CHAR_H) % (12*14),"browse");
			else
				print_str((6+7+1)*CHAR_W,(LINE_STATE*CHAR_H) % (12*14),"      ");
		}


		//----------------
		// Event handling
		// ---------------
		switch (player.last_event) {

		case EV_NONE:
			break;

		case EV_EOF:
			// EV_EOF means that a song was played completely
			// close file and go back to IDLE state
			// close file
			uart_puts("finished.\n\r");
			//close is not required for FatFs, just set player state
			// go to idle state
			player.state&=(~S_PLAY);
			player.state|=S_IDLE;
			break;

		case EV_NEXT:
			// EV_NEXT means that user wants immediatedly to cue to next song
			// reaction is easy: close file and go back to IDLE state
			// just set state according, rest is done in idle state handling
			uart_puts("go to next song.\n\r");
			// go to idle state
			player.state&=(~S_PLAY);
			player.state|=S_IDLE;
			break;

		case EV_RESTART:
			// EV_RESTART means that user wants immediatedly to restart current song
			// reaction is easy: close file, open again, remain in PLAY state
			// just open file again
			uart_puts("restart song.\n\r");
			ret = f_open( &fil, g_path_file, FA_READ );
			if(ret != FR_OK) {
				uart_puts("error opening ");
				uart_puts("file\n\r");
			}
			break;

		case EV_BROWSE_OK:
			// enter browsing state
			// file can be browsed (EV_DOWN) and selected for play (EV_RIGHT)
			if (!(player.state&S_BROWSE)) {
				// entering browsing state
				player.state |= S_BROWSE;
				// init subdir path
				strcpy( g_browse_dir, g_path_dir );
				uart_puts("will browse files in: ");
				uart_puts(g_browse_dir );
				uart_puts("\n\r");
				ret = f_opendir( &browse_dir, g_browse_dir );
				if (ret!=FR_OK) {
					uart_puts("f_opendir failed\n\r");
				} else {
					uart_puts("f_opendir ok\n\r");
				}
			}
			else {
				// leaving browsing state
				// entering browsing state
				player.state &= ~S_BROWSE;
				uart_puts("stopping browse files.\n\r");
			}
			break;

		case EV_BROWSE_DOWN:
			uart_puts("browsing object: ");
			ret = f_readdir(&browse_dir, &browse_info);
			if(ret != FR_OK) {
				uart_puts("error reading dir 1 ");
			} else {
				uart_puts("f_readdir ok\n\r");
			}
			uart_puts(browse_info.fname);
			uart_puts("\n\r");
			if (browse_info.fattrib & AM_DIR) {
				sprintf( sbuf, "<<< %s >>>", browse_info.fname );
				//uart_puts("directory: ");
				//uart_puts(browse_info.fname);
			}
			if (!(browse_info.fattrib & AM_DIR)) {
				sprintf( sbuf, "%s", browse_info.fname );
				//uart_puts("mp3 file:  ");
				//uart_puts(browse_info.fname);
			}
			//uart_puts(sbuf);
			//uart_puts("\n\r");
			print_str(0,(LINE_BROWSE*CHAR_H) % (12*14),"                    ");
			sbuf[21] =0;
			print_str(0,(LINE_BROWSE*CHAR_H) % (12*14),sbuf);
			break;

		case EV_BROWSE_RIGHT:
			//if (prevent_double_select)
			//	break;
			//prevent_double_select++;
			// selection can be on a) mp3 files and b) directories
			if (!(browse_info.fattrib  & AM_DIR) && (strstr(browse_info.fname, ".MP3") != NULL) ) {
				// a mp3 file was selected
				// get current path
				// add newly selected FILE to path
				strcpy( g_browse_file, g_browse_dir );
				// for root dir string "/", we don't want to append '/'
				//if (!strcmp(g_browse_file,"/")) { /* strcmp does not work here, hm, hm ... */
				if (strlen(g_browse_file)>1) {
					//uart_puts("append a slash!!!\n\r");
					strcat( g_browse_file,  "/" );
				}
				strcat( g_browse_file,  browse_info.fname );
				uart_puts("selecting file object: \"");
				uart_puts( g_browse_file);
				uart_puts("\"\n\r");
				// at LCD , print only file name
				print_str(0,(LINE_ARTIST*CHAR_H) % (12*14),browse_info.fname);

				// try to open selected file
				ret = f_open( &browse_fil,  g_browse_file, FA_READ );
				if(ret != FR_OK) {
					uart_puts("error opening ");
					uart_puts("file\n\r");
				}

				print_id3_tags( &browse_fil );

				// play song (see S_PLAY handling above)
				uart_puts("try to play mp3\n\r");
				player.state |= S_PLAY;
				player.state&=(~S_IDLE);

				// adjust file vars
				memcpy(&fil, &browse_fil, sizeof(fil));
				// adjust dir vars next etc. start fro new dir
				memcpy(&dir, &browse_dir, sizeof(dir));

				// setup path variable for fopen of file
				strcpy( g_path_dir, g_browse_dir );
				uart_puts("adjusting path to: ");
				uart_puts(g_path_dir);
				uart_puts("\n\r");

			}

			if (browse_info.fattrib  & AM_DIR) {
				// a dir was selected
				uart_puts("subdir ");
				// enlarge dir var by this new dir part
				// delimiter only for non root dir "/"
				if (strlen(g_browse_dir)>1)
					strcat( g_browse_dir,  "/" );
				strcat( g_browse_dir, browse_info.fname );
				uart_puts("selecting dir object: ");
				uart_puts(g_browse_dir);
				uart_puts("\n\r");
				// try to open dir
				ret = f_opendir( &browse_dir, g_browse_dir );
				if (ret!=FR_OK) {
					uart_puts("f_opendir failed\n\r");
				} else {
					uart_puts("f_opendir ok\n\r");
				}
			}
			break;

		case EV_BROWSE_LEFT:
			// back in dir hierarchy: delete away one path path, e.g. B in /A/B
			// find first '/' from the right
			p = strrchr( g_browse_dir, '/' );
			// if '/' was found ...
			if (p) {
				if (p==g_browse_dir) {
					// we reached the root slash. don't delete it,
					// terminate string sirectly behind it and leave "/"
					*(++p) = '\0';
				} else {
					// terminate string "A/B" at '/' and leave "/A"
					*p = '\0';
				}
			}
			uart_puts("adjusting path to: ");
			uart_puts(g_browse_dir);
			uart_puts("\n\r");

			uart_puts("now selecting dir object: ");
			uart_puts(g_browse_dir);
			uart_puts("\n\r");
			// try to open dir
			ret = f_opendir( &browse_dir, g_browse_dir );
			if (ret!=FR_OK) {
				uart_puts("f_opendir failed\n\r");
			} else {
				uart_puts("f_opendir ok\n\r");
			}
			break;

		default:
			uart_puts("unknown event\n\r");
			break;
		}
		// always clear event code
		player.last_event=EV_NONE;

		//----------------
		// RS232 input processing
		// ---------------
		//input_process();

		//----------------
		// IR input processing (remote control)
		// ---------------
		if (-1 != rc5.flip) {
			// get code and addr from rc5 global variable, filled by ISR
			// note: I ignore addr value; every remote control is acceppted
			// hopefully codes are euqally between IRs
			ir_code = rc5.code;
			ir_addr = rc5.addr;
			// clear flip value for next key code from remote control
			rc5.flip = -1;

			/* put out info; addr is "address" of remote control, code is key code */
			sprintf(sbuf, "IR %02x sent: %02x.\n\r", ir_addr, ir_code);
			uart_puts(sbuf);

			// command loop
			switch (ir_code) {
			case 0x1e:
				// key: >| (next)
				// go to next file
				player.last_event = EV_NEXT;
				break;
			case 0x30 :
				// key: || (pause)
				if (player.state&S_PLAY) {
					player.state |= S_PAUSE;
					player.state &= ~S_PLAY;
				}
				else {
					player.state |= S_PLAY;
					player.state &= ~S_PAUSE;
				}
				break;
			case 0x35:
				// key: > (play)
				player.state |= S_PLAY;
				break;
			case 0x11:
				// key: Volume-
				if (player.volume<=(255-5)) {
					player.volume +=5;
					vs1011_dim_volume(player.volume);
				}
				break;
			case 0x10:
				// key: Volume+
				if (player.volume>=5) {
					player.volume -=5;
					vs1011_dim_volume(player.volume);
				}
				break;
			case 0x32:
				// key: << (replay)
				// ignore command if not in play state
				if (player.state&S_PLAY)
					player.last_event = EV_RESTART;
				break;


				// DIRECTORY BROWSING EVENTS
			case 0x14:
				// key: ^ cursor-up
				// ignore command if not in browse state
				//prevent_double_select=0;
				if (player.state&S_BROWSE)
					player.last_event = EV_BROWSE_UP;
				break;
			case 0x15:
				// key: v cursor-down
				// ignore command if not in browse state
				//prevent_double_select=0;
				if (player.state&S_BROWSE)
					player.last_event = EV_BROWSE_DOWN;
				break;
			case 0x16:
				// key: < cursor-left
				// ignore command if not in browse state
				//prevent_double_select=0;
				if (player.state&S_BROWSE)
					player.last_event = EV_BROWSE_LEFT;
				break;
			case 0x17:
				// key: > cursor-right
				// ignore command if not in browse state
				if (player.state&S_BROWSE)
					player.last_event = EV_BROWSE_RIGHT;
				break;
			case 0x25:
				// key: cursor-OK
				//prevent_double_select=0;
				player.last_event = EV_BROWSE_OK;
				break;




			default:
				uart_puts("pixels");
				draw();
				break;
			}
		}

	}
}

//#define LOGIC_DEBUGGING
#ifdef LOGIC_DEBUGGING
/**
 * \ingroup debug
 * debug utility function. Dumps a buffer in hex and ascii.
 *
 * \param[in] ptr - pointer to buffer to print
 * \param[in] size - number of bytes to print
 */
void display_buffer( uint8_t * ptr, uint32_t size) {
	uint32_t i;
	uint16_t j;
	uint8_t *local;
	char sbuf[16];

	local = ptr;
	for (i=0; i < size; i++) {
		if (!(i % 16)) {
			if (i) {
				for (j=0;j<16;j++) {
					if (*local < ' ')
						uart_putc('.');
					else
						uart_putc(*local);
					local++;
				}
			}
			//printf("\r\n%4X - ",i);
			sprintf( sbuf, "\r\n%4X - ", (unsigned int) i );
			uart_puts( sbuf );
		}
		//uart_putc_hex(*ptr++);
		//uart_putc(' ');
		sprintf( sbuf, "%02x ", *ptr++ );
		uart_puts( sbuf );
		//printf("%X ",*ptr++);
	}

	for (j=0;j<16;j++) 	{
		if (*local < ' ')
			uart_putc('.');
		else
			uart_putc(*local);
		local++;
	}
	uart_puts("\r\n");
}
#endif


#ifdef SERIAL_INPUT
void input_process( void ) {
	unsigned int c;
	//char buf[10];

	/*
	 * Get received character from ringbuffer
	 * uart_getc() returns in the lower byte the received character and
	 * in the higher byte (bitmask) the last receive error
	 * UART_NO_DATA is returned when no data is available.
	 *
	 */
	c = uart_getc();
	//c = getc(stdin);
	if ( c & UART_NO_DATA ) {
		// no data available from UART
		return;
	}

	/*
	 * new data available from UART
	 * check for Frame or Overrun error
	 */
	if ( c & UART_FRAME_ERROR ) {
		/* Framing Error detected, i.e no stop bit detected */
		uart_puts_P("UART Frame Error: ");
	}
	if ( c & UART_OVERRUN_ERROR ) {
		/*
		 * Overrun, a character already present in the UART UDR register was
		 * not read by the interrupt handler before the next character arrived,
		 * one or more received characters have been dropped
		 */
		uart_puts_P("UART Overrun Error: ");
	}
	if ( c & UART_BUFFER_OVERFLOW ) {
		/*
		 * We are not reading the receive buffer fast enough,
		 * one or more received character have been dropped
		 */
		uart_puts_P("Buffer overflow error: ");
	}

	/*
	 * send received character back
	 */
	uart_puts("You entered: ");
	uart_putc( (unsigned char)c );
	uart_puts("\n\r");

	// command loop
	switch (c) {
	case 'n' :
		// go to next file
		player.last_event = EV_NEXT;
		break;
	case 'p' :
		if (player.state==S_PLAY)
			player.state = S_PAUSE;
		else
			player.state = S_PLAY;
	default:
		break;
	}
}
#endif
