/*************************************************************************
Author:   	$Author: dennis $
File:     	$HeadURL: file:///home/dennis/svn-store/avr-source/mptry-fat32/spi.h $
Date:  		$Date: 2008-10-11 19:18:49 +0200 (Sa, 11. Okt 2008) $ 
Revision: 	$Revision: 142 $ 
Id: 		$Id: spi.h 142 2008-10-11 17:18:49Z dennis $ 
Licence:	GNU General Public License
 *************************************************************************/
/** 
 *  @defgroup spi SPI master implementation
 *  @code #include <spi.h> @endcode
 * 
 *  @brief SPI master implementation for AVR
 *
 *  SPI master implementation. Some functions for easy use of spi functionality of AVR. 
 *
 *  @author Dennis Dingeldein  http://www.dingeldein-online.de
 *
 *
 * \section Licence
 * This code can be used according to GNU General Public License.
 * 
 * \author 
 * Dennis Dingeldein
 */

/**
 * \addtogroup spi
 *
 * @{
 */
/**
 * \file
 * SPI master implementation
 */

#ifndef SPI_H_
#define SPI_H_

#include <stdint.h>


/* defines for customization of spi access */
#if defined(__AVR_ATmega8__) || \
    defined(__AVR_ATmega48__) || \
    defined(__AVR_ATmega88__) || \
    defined(__AVR_ATmega168__)
    #define configure_pin_mosi() DDRB |= (1 << DDB3)
    #define configure_pin_sck() DDRB |= (1 << DDB5)
    #define configure_pin_ss() DDRB |= (1 << DDB2)
    #define configure_pin_miso() DDRB &= ~(1 << DDB4)
#elif defined(__AVR_ATmega16__) || \
      defined(__AVR_ATmega32__) || \
      defined(__AVR_ATmega644__) || defined(__AVR_ATmega644P__)
    #define configure_pin_mosi() DDRB |= (1 << DDB5)
    #define configure_pin_sck() DDRB |= (1 << DDB7)
    #define configure_pin_ss() DDRB |= (1 << DDB4)
    #define configure_pin_miso() DDRB &= ~(1 << DDB6)
#elif defined(__AVR_ATmega64__) || \
      defined(__AVR_ATmega128__) || \
      defined(__AVR_ATmega169__)
    #define configure_pin_mosi() DDRB |= (1 << DDB2)
    #define configure_pin_sck() DDRB |= (1 << DDB1)
    #define configure_pin_ss() DDRB |= (1 << DDB0)
    #define configure_pin_miso() DDRB &= ~(1 << DDB3)
#else
    #error "no spi pin mapping available!"
#endif


extern uint8_t spi_init();
extern uint8_t spi_set_maxspeed();
extern uint8_t spi_read_byte();
extern uint8_t spi_write_byte( int8_t byte );

/**
 * @}
 */

#endif /*SPI_H_*/
