/*************************************************************************
Author:   	$Author: dennis $
File:     	$HeadURL: file:///home/dennis/svn-store/avr-source/gpib_004/timer16.c $
Date:  		$Date: 2008-05-11 07:57:02 +0200 (So, 11 Mai 2008) $
Revision: 	$Revision: 65 $
Id: 		$Id: timer16.c 65 2008-05-11 05:57:02Z dennis $
Licence:	GNU General Public License
 *************************************************************************/

/**
 *  @defgroup timer_lib Timer Library
 *  @code #include <timer16.h> @endcode
 *
 *  @brief Timer Library.
 *
 *  16 bit timer implementation.
 *
 *  @author Dennis Dingeldein  http://www.dingeldein-online.de
 */

#include "timer16.h"

#if defined(__AVR_ATmega644__) || \
	defined(__AVR_ATmega644P__)
    #define TCCR0 	TCCR0A
    #define GIFR  EIFR
    #define GICR	EIMSK
    #define TIMSK	TIMSK1
    #define TIFR	TIFR0
#endif

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <stdio.h>
#include "uart.h"

#include "vs1011.h"
#include "player.h"
#include "spi.h"

volatile uint16_t timer; /**< timer value, increased by interrupt. Not to be used by user functions. */
volatile uint8_t s; /**< second value, increased by one every second. To be used (RW) by user functions for clock purposes. */
volatile uint16_t UserTimer1;  /**< user timer value, increased by interrupt. To be used (RW) by user functions. */
/**
 * Interrupt Service Routine - do not call directly
 *  This routine is called when the Timer Value TCNT1 reaches the Output Compare Register Value OCR1A
 */
ISR(TIMER1_COMPA_vect) {

#if F_CPU % DEBOUNCE
	OCR1A = F_CPU / DEBOUNCE - 1;
#endif
	if (--timer==0) {
		timer=(uint16_t) (F_CPU / DEBOUNCE) ;
		s++;
#if F_CPU % DEBOUNCE
		OCR1A = F_CPU / DEBOUNCE + F_CPU % DEBOUNCE - 1;
#endif
	}
}

/**
 * Timer initialisation.
 *  \brief initializes all registers ands sets second value 's' to zero. Timer is immediately
 * started.
 */
void timer16_init( void ) {
	// Initialisierung:
	// (1<<CS10) : Timer1 Vorteiler = 001 = 1. Der Zähler wird also mit f=16Mhz hochgezählt

	// OCR1A=XTAL/DEBOUNCE-1 -> Bei Erreichen dieses Wertes (4000) wird die ISR besucht
	// die ISR wird also alle 1/4000s aufgerufen. Wenn man dort also bis 4000 hochzählt, ist genau
	// 1 Sekunde um!

	TCCR1B = (1<<CS10) ^ (1<<WGM12);	// Prescaler of 1 | CTC mode
	OCR1A  = (F_CPU/DEBOUNCE)-1;		// Output compare register value
	TCNT1 = 0; // Start value for timer register
	s=0; // Initialize second value (s) to zero
	timer = (uint8_t)DEBOUNCE;
	TIMSK |= (1<<OCIE1A);		// activate timer interrupts which starts timer run


}
