/*************************************************************************
Author:   	$Author: dennis $
File:     	$HeadURL: file:///home/dennis/svn-store/avr-source/mptry-1/spi.h $
Date:  		$Date: 2008-09-06 10:51:00 +0200 (Sa, 06 Sep 2008) $ 
Revision: 	$Revision: 94 $ 
Id: 		$Id: spi.h 94 2008-09-06 08:51:00Z dennis $ 
Licence:	GNU General Public License
 *************************************************************************/
/** 
 *  @defgroup vs1011 MP3 decoder implementation
 *  @code #include <vs1011.h> @endcode
 * 
 *  @brief vs1011 MP3 decoder implementation for AVR
 *
 *  vs1011 MP3 decoder implementation. Some functions for easy use of vs1011. 
 *
 *  @author Dennis Dingeldein  http://www.dingeldein-online.de
 *
 *
 * \section Licence
 * This code can be used according to GNU General Public License.
 * 
 * \author 
 * Dennis Dingeldein
 */

/**
 * \addtogroup vs1011
 *
 * @{
 */
/**
 * \file
 * vs1011 MP3 decoder implementation.
 * vs1011 is accessed in its "SHARED and NEW MODE", See VS1011 manual for that.
 * 
 * For use, the defines XTAL and VS1011_* must be set up according your hardware configuration.  
 */

#ifndef VS1011_H_
#define VS1011_H_

#include <stdint.h>
#include <avr/io.h>

/* define NON_SHARED to not use SHARED MODE, then XDCS must be used for SDI mode */
#define NON_SHARED

/* external XTAL frequency of VS10xx */
#define VS_XTAL 12288000L

/* value for SCI_CLOCKF; add 0x0800 if clock doubling is to be used */
#define XVAL ((VS_XTAL/2000L)+0x8000L)

/* pin connectivity between VS10xx and AVR */
#define VS1011_INPORT		PINA
#define VS1011_XCS 		PA1
#define VS1011_XRESET		PA2
#define VS1011_DREQ		PA3
#ifdef NON_SHARED
#define VS1011_XDCS		PA4
#endif

#define VS_LOOP_UNTIL_DREQ_HIGH  loop_until_bit_is_set(VS1011_INPORT,VS1011_DREQ)

extern uint8_t vs1011_port_init( void ) ;
uint8_t vs1011_init( void ) ;
uint8_t vs1011_sw_reset( void );
extern void vs1011_xcs_select( void ) ;
extern void vs1011_xcs_deselect( void ) ;
extern void vs1011_xdcs_select( void ) ;
extern void vs1011_xdcs_deselect( void ) ;

uint8_t vs1011_dim_volume( uint8_t v );

extern uint8_t vs1011_test_vol( void ) ;

#endif /*VS1011_H_*/
